
/*
 * This file contain methods that will be used to 
 * manipulate the LCD screen.
 *
 * @author Branden Archer
 */

#ifndef __LCDLIBRARY_H_
#define __LCDLIBRARY_H_

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "BasicStamp2Library.h"

#define charactersPerLine 20
#define maxVarNameSize charactersPerLine-5

//The LCD and the accompanying shift register are 
//considered one large entity (as they share each
//other's I/O lines, and if they are combined
//it may save in some code space).


//On this processor, if we run at 1MHz, that is one
//instruction per u-second. Many of the timing 
//requirements of the LCD screen are measured
//in nano-seconds. There is no need for
//sleep commands to execute commands,
//but to make sure the command is executed before
//returning, a sleep statement may be necessary.


//When the address of the LCD is set,
//bit7 must be a 1. This is the
//set DDRAM command. CGRAM deals with
//the custom characters
#define setLcdAddrCommand 0x80
#define line1Addr 0x0+setLcdAddrCommand
#define line2Addr 0x40+setLcdAddrCommand
#define line3Addr 0x14+setLcdAddrCommand
#define line4Addr 0x54+setLcdAddrCommand

#define clearLcdCommand 0x01



//bits 0,2 are used for the LCD.
//do PINC & clearLcdBits to clear
//these bits
#define clearLcdBits 0xFA
#define enable 0x04
//is rs bit set to 1
//rs is pin0
#define writeCommandPinCombo 0x01
#define systemCommandPinCombo 0x00
#define setAddrCommandPinCombo 0x00
//On this processor, if we run at 1MHz, that is one
//instruction per u-second. Many of the timing 
//requirements of the LCD screen are measured
//in nano-seconds. There is no need for
//sleep commands to execute commands,
//but to make sure the command is executed before
//returning, a sleep statement may be necessary.

//setup variables
#define initSetupBitPatten  0x30

#define setFunctionCommand  0x20
#define bitInterface4       0x00
#define bitInterface8       0x10
#define lineDisplay1        0x00
#define lineDisplay2        0x08
#define Font5x10            0x04
#define Font5x7             0x00

#define setEntryModeCommand 0x04
#define cursorIncreases     0x02
#define cursorDecreases     0x00
#define displayShifted      0x01
#define displayNotShifted   0x00

#define setDisplayCommand   0x08
#define turnDisplayOff      0x00
#define turnDisplayOn       0x04
#define turnCursorOn        0x02
#define turnCursorOff       0x00
#define BlinkCursor         0x01
#define noBlinkCursor       0x00





/*
 * Initializes the Shift register, then the
 * LCD display.
 */
void initLcd();

/*
 * Clears the LCD display by clearing out
 * every character on the display.
 */
void clearLcd();

/*
 * This sets line 1 of the LCD display to a certain
 * character string. Each line holds 20 characters.
 * Each characters must be specified. The last character
 * is assumed to be a null, but in reality it is ignored.
 * If 20 characters are not present, random things may be
 * written to the display.
 *
 * @param line - line of 20 characters to write to LCD
 */
void setLcdLine1(char line [charactersPerLine + 1]);


/*
 * This sets line 2 of the LCD display to a certain
 * character string. Each line holds 20 characters.
 * Each characters must be specified. The last character
 * is assumed to be a null, but in reality it is ignored.
 * If 20 characters are not present, random things may be
 * written to the display.
 *
 * @param line - line of 20 characters to write to LCD
 */
void setLcdLine2(char line [charactersPerLine + 1]);

/*
 * This sets line 1 of the LCD display to a certain
 * character string. Each line holds 20 characters.
 * Each characters must be specified. The last character
 * is assumed to be a null, but in reality it is ignored.
 * If 20 characters are not present, random things may be
 * written to the display.
 *
 * @param line - line of 20 characters to write to LCD
 */
void setLcdLine3(char line [charactersPerLine + 1]);

/*
 * This sets line 1 of the LCD display to a certain
 * character string. Each line holds 20 characters.
 * Each characters must be specified. The last character
 * is assumed to be a null, but in reality it is ignored.
 * If 20 characters are not present, random things may be
 * written to the display.
 *
 * @param line - line of 20 characters to write to LCD
 */
void setLcdLine4(char line [charactersPerLine + 1]);

/*
 * Creates a debug line for the display that will convert
 * the variable to a string, then combine the varName field
 * and the variable. The result will look something like this:
 * varName=value
 * If the combonation of the buffer and varName is more than
 * 20 characters, the end of the variable's value will be
 * concatinated, and the last character in the row will be '_'.
 *
 * @param buffer - Where the result will be stored
 * @param varName - the name of the variable. The end of the name
 *                  should be marked with a null.
 * @param variable - the variable that will be converted to a
 *                   string. You can type cast bytes for this 
 *                   field.
 * @param numberBase - the number base the string will be in.
 *                     common bases are 2, 10, and 16. In the
 *                     string, binary will be denoted by 0b, 
 *                     and hex will be denoted by 0x
 */
void createDebugLine(char buffer [charactersPerLine + 1], char varName [maxVarNameSize + 1], int variable, int numberBase);

#define maxCharsPlusNull charactersPerLine+1

/*
 * All the string tables are compressed based on the largest
 * string in that table. This method will convert the string
 * to the correct string size.
 *
 * @param input - the input string to format to the correct 
 *        size
 * @param inputSize - the size of the input string
 * @param output - an already allocated output buffer of 
 *        21 characters that the resultant string will be
 *        stored in.
 */
void formTo21Chars(char * input, int inputSize, char * output);


/*
 * Formats the input string to the correct length, then
 * outputs it to the desired LCD line. Invalid lines
 * are ignored
 *
 * @param line - line to write to
 * @param toWrite - the line to write
 * @param size - the size of the line including null
 */
void formatAndSendToLcd(int line, char * toWrite, int size);


/*
 * Find the size of the string toWrite then sends it
 * over for formatting and sending. If you know the
 * size of the string use formatAndSendToLcd instead.
 *
 * @param line - what line to write to
 * @param toWrite - line to write
 */
void sendToLcd(int line, char * toWrite);


/*
 * THIS SHOULD NOT BE CALLED DIRECTLY
 * Shift in a byte of data into the shift register.
 * Data is shifted in LSB first. This means that
 * Shift reg bit 7 -> LCD bit 0.
 *
 * @param data
 */
void shiftIn(byte data);

/*
 * THIS SHOULD NOT BE CALLED DIRECTLY
 * For sending the characters over to the LCD, if a null
 * character is being sent, it must be converted to
 * a space for the display.
 *
 * @param line - line to convert nulls to spaces in
 */
 void convNullsToSpaces(char line[charactersPerLine+1]);

/*
 * THIS SHOULD NOT BE CALLED DIRECTLY
 * (To save in code space), this method will
 * shift in data to the shift register, then
 * using the shift register send that data to 
 * the LCD.
 *
 * @param data - data to send to LCD
 * @param linesToRaise - what bit pattern should
 * be on PORTC to do the communication
 * (ie, the function that is being executed).
 */
void sendData(byte data, byte linesToRaise);

/*
 * THIS SHOULD NOT BE CALLED DIRECTLY
 * Writes 20 characters to the LCD starting at
 * the LCD's current address.
 *
 * @param line - line to write
 */
void writeToLcd(char line[charactersPerLine+1]);

#endif
